@@ -90,6 +90,63 @@ namespace xt
     }
 #endif
 
+    /**
+     * Create a xcontainer (xarray, xtensor or xtensorf) with uninitialized values of
+     * the same shape, value type and layout as the input xexpression *e*.
+     *
+     * @param e the xexpression from which to extract shape, value type and layout.
+     */
+    template <class E>
+    inline typename E::temporary_type empty_like(const xexpression<E>& e)
+    {
+        typename E::temporary_type res(e.derived_cast().shape());
+        return res;
+    }
+
+    /**
+     * Create a xcontainer (xarray, xtensor or xtensorf), filled with *fill_value* and of
+     * the same shape, value type and layout as the input xexpression *e*.
+     *
+     * @param e the xexpression from which to extract shape, value type and layout.
+     * @param fill_value the value used to set each element of the returned xcontainer.
+     */
+    template <class E>
+    inline typename E::temporary_type full_like(const xexpression<E>& e, typename E::value_type fill_value)
+    {
+        typename E::temporary_type res(e.derived_cast().shape(), fill_value);
+        return res;
+    }
+
+    /**
+     * Create a xcontainer (xarray, xtensor or xtensorf), filled with zeros and of
+     * the same shape, value type and layout as the input xexpression *e*.
+     *
+     * Note: contrary to zeros(shape), this function returns a non-lazy, allocated container! 
+     * Use ``xt::zeros<double>(e.shape());` for a lazy version.
+     *
+     * @param e the xexpression from which to extract shape, value type and layout.
+     */
+    template <class E>
+    inline typename E::temporary_type zeros_like(const xexpression<E>& e)
+    {
+        return full_like(e, typename E::value_type(0));
+    }
+
+    /**
+     * Create a xcontainer (xarray, xtensor or xtensorf), filled with ones and of
+     * the same shape, value type and layout as the input xexpression *e*.
+     *
+     * Note: contrary to ones(shape), this function returns a non-lazy, evaluated container!
+     * Use ``xt::ones<double>(e.shape());`` for a lazy version.
+     *
+     * @param e the xexpression from which to extract shape, value type and layout.
+     */
+    template <class E>
+    inline typename E::temporary_type ones_like(const xexpression<E>& e)
+    {
+        return full_like(e, typename E::value_type(1));
+    }
+
     namespace detail
     {
         template <class T>
