@@ -165,7 +165,7 @@ namespace xt
      * xindexed_stepper *
      ********************/
 
-    template <class E, bool is_const = true>
+    template <class E, bool is_const>
     class xindexed_stepper
     {
     public:
@@ -515,6 +515,7 @@ namespace xt
         }
         if (i == 0)
         {
+            std::copy(shape.cbegin(), shape.cend(), index.begin());
             stepper.to_end(layout_type::row_major);
         }
     }
@@ -560,6 +561,7 @@ namespace xt
         }
         if (i == 0)
         {
+            std::copy(shape.cbegin(), shape.cend(), index.begin());
             stepper.to_end(layout_type::row_major);
         }
     }
@@ -670,6 +672,7 @@ namespace xt
         }
         if (i == size)
         {
+            std::copy(shape.cbegin(), shape.cend(), index.begin());
             stepper.to_end(layout_type::column_major);
         }
     }
@@ -717,6 +720,7 @@ namespace xt
         }
         if (i == size)
         {
+            std::copy(shape.cbegin(), shape.cend(), index.begin());
             stepper.to_end(layout_type::column_major);
         }
     }
